@@ -62,7 +62,9 @@
 #include <syslog.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <sys/mman.h>
 #include <pwd.h>
+#include <grp.h>
 
 /*
  * PAM Preamble
@@ -72,7 +74,6 @@
 #define PAM_SM_SESSION
 
 #include <security/pam_modules.h>
-#include <security/pam_modutil.h>
 #include <security/pam_ext.h>
 
 /*
@@ -353,14 +354,15 @@ static int cdbus_create_snapshot( const char *snapper_conf, createmode_t createm
  * Special functions for pam_snapper
  */
 
-static int forker( pam_handle_t * pamh, uid_t uid, gid_t gid, const char *snapper_conf,
-		   createmode_t createmode, const char *cleanup, uint32_t num_user_data,
-		   const struct dict *user_data, const uint32_t * snapshot_num_in,
-		   uint32_t * snapshot_num_out )
+static int forker( pam_handle_t * pamh, const char *pam_user, uid_t uid, gid_t gid,
+		   const char *snapper_conf, createmode_t createmode, const char *cleanup,
+		   uint32_t num_user_data, const struct dict *user_data,
+		   const uint32_t * snapshot_num_in, uint32_t * snapshot_num_out )
 {
-	int fds[2];
-	if ( pipe( fds ) != 0 ) {
-		pam_syslog( pamh, LOG_ERR, "pipe failed" );
+	void *p = mmap( NULL, sizeof( *snapshot_num_out ), PROT_READ | PROT_WRITE,
+			MAP_SHARED | MAP_ANONYMOUS, -1, 0 );
+	if ( p == MAP_FAILED ) {
+		pam_syslog( pamh, LOG_ERR, "mmap failed" );
 		return -1;
 	}
 
@@ -369,59 +371,49 @@ static int forker( pam_handle_t * pamh, uid_t uid, gid_t gid, const char *snappe
 
 		/* setting uid/gui affects other threads so it has to be done in a separate process */
 
-		if ( setegid( gid ) != 0 || seteuid( uid ) != 0 ) {
+		if ( setgid( gid ) != 0 || initgroups( pam_user, gid ) != 0 || setuid( uid ) != 0 ) {
+			munmap( p, sizeof( *snapshot_num_out ) );
 			exit( EXIT_FAILURE );
 		}
 
-		close( fds[0] );
-
 		if ( cdbus_create_snapshot( snapper_conf, createmode, cleanup, num_user_data,
-					    user_data, snapshot_num_in, snapshot_num_out ) != 0 )
+					    user_data, snapshot_num_in, snapshot_num_out ) != 0 ) {
+			munmap( p, sizeof( *snapshot_num_out ) );
 			exit( EXIT_FAILURE );
+		}
 
-		if ( pam_modutil_write
-		     ( fds[1], ( char * )snapshot_num_out,
-		       sizeof( *snapshot_num_out ) ) != sizeof( *snapshot_num_out ) )
-			exit( EXIT_FAILURE );
+		memcpy( p, snapshot_num_out, sizeof( *snapshot_num_out ) );
 
-		close( fds[1] );
+		munmap( p, sizeof( *snapshot_num_out ) );
 
 		exit( EXIT_SUCCESS );
 
 	} else if ( child > 0 ) {
 
-		close( fds[1] );
-
 		int status;
 		int ret = waitpid( child, &status, 0 );
 
 		if ( ret == -1 ) {
 			pam_syslog( pamh, LOG_ERR, "waitpid failed" );
-			close( fds[0] );
+			munmap( p, sizeof( *snapshot_num_out ) );
 			return -1;
 		}
 
 		if ( !WIFEXITED( status ) ) {
 			pam_syslog( pamh, LOG_ERR, "child exited abnormal" );
-			close( fds[0] );
+			munmap( p, sizeof( *snapshot_num_out ) );
 			return -1;
 		}
 
 		if ( WEXITSTATUS( status ) != EXIT_SUCCESS ) {
 			pam_syslog( pamh, LOG_ERR, "child exited normal but with failure" );
-			close( fds[0] );
+			munmap( p, sizeof( *snapshot_num_out ) );
 			return -1;
 		}
 
-		if ( pam_modutil_read
-		     ( fds[0], ( char * )snapshot_num_out,
-		       sizeof( *snapshot_num_out ) ) != sizeof( *snapshot_num_out ) ) {
-			pam_syslog( pamh, LOG_ERR, "reading snapshot_num_out failed" );
-			close( fds[0] );
-			return -1;
-		}
+		memcpy( snapshot_num_out, p, sizeof( *snapshot_num_out ) );
 
-		close( fds[0] );
+		munmap( p, sizeof( *snapshot_num_out ) );
 
 		return 0;
 
@@ -492,6 +484,10 @@ static int worker( pam_handle_t * pamh, const char *pam_user, const char *snappe
 
 	uint32_t *snapshot_num_in = NULL;
 	uint32_t *snapshot_num_out = malloc( sizeof( *snapshot_num_out ) );
+	if ( !snapshot_num_out ) {
+		pam_syslog( pamh, LOG_ERR, "out of memory" );
+		return -1;
+	}
 
 	if ( createmode == createmode_post ) {
 		if ( pam_get_data
@@ -502,8 +498,8 @@ static int worker( pam_handle_t * pamh, const char *pam_user, const char *snappe
 		}
 	}
 
-	if ( forker( pamh, uid, gid, snapper_conf, createmode, cleanup, num_user_data, user_data,
-		     snapshot_num_in, snapshot_num_out ) != 0 )
+	if ( forker( pamh, pam_user, uid, gid, snapper_conf, createmode, cleanup, num_user_data,
+		     user_data, snapshot_num_in, snapshot_num_out ) != 0 )
 		return -1;
 
 	if ( pam_set_data
@@ -661,8 +657,10 @@ static char *get_snapper_conf( const char *pam_user, const pam_options_t * optio
 		snapper_conf = strdup( "root" );
 	} else {
 		snapper_conf = malloc( strlen( options->homeprefix ) + strlen( pam_user ) + 1 );
-		strcpy( snapper_conf, options->homeprefix );
-		strcat( snapper_conf, pam_user );
+		if ( snapper_conf ) {
+			strcpy( snapper_conf, options->homeprefix );
+			strcat( snapper_conf, pam_user );
+		}
 	}
 	return snapper_conf;
 }
@@ -690,6 +688,7 @@ static void pam_session( pam_handle_t * pamh, openclose_t openclose, int argc, c
 
 	char *snapper_conf = get_snapper_conf( pam_user, &options );
 	if ( !snapper_conf ) {
+		pam_syslog( pamh, LOG_ERR, "out of memory" );
 		return;
 	}
 
