@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012 Novell, Inc.
+ * Copyright (c) [2012-2013] Novell, Inc.
  *
  * All Rights Reserved.
  *
@@ -885,8 +885,8 @@ Client::create_post_snapshot(DBus::Connection& conn, DBus::Message& msg)
     snap2->setUserdata(userdata);
     snap2->flushInfo();
 
-    map<string, string>::const_iterator pos = it->config_info.raw.find("BACKGROUND_COMPARISON");
-    if (pos == it->config_info.raw.end() || pos->second == "yes")
+    bool tmp;
+    if (it->config_info.getValue("BACKGROUND_COMPARISON", tmp) && tmp)
 	backgrounds.add_task(it, snap1, snap2);
 
     DBus::MessageMethodReturn reply(msg);
