@@ -68,6 +68,7 @@ namespace snapper
 	    y2err("not a directory path:" << base_path);
 	    throw IOErrorException();
 	}
+
 #ifdef ENABLE_XATTRS
 	setXaStatus();
 #endif
@@ -95,6 +96,7 @@ namespace snapper
 	    close(dirfd);
 	    throw IOErrorException();
 	}
+
 #ifdef ENABLE_XATTRS
 	xastatus = dir.xastatus;
 #endif
@@ -104,12 +106,13 @@ namespace snapper
     SDir::SDir(const SDir& dir)
 	: base_path(dir.base_path), path(dir.path)
     {
-	dirfd = dup(dir.dirfd);
+	dirfd = fcntl(dir.dirfd, F_DUPFD_CLOEXEC, 0);
 	if (dirfd == -1)
 	{
-	    y2err("dup failed" << " error:" << stringerror(errno));
+	    y2err("fcntl(F_DUPFD_CLOEXEC) failed error:" << stringerror(errno));
 	    throw IOErrorException();
 	}
+
 #ifdef ENABLE_XATTRS
 	xastatus = dir.xastatus;
 #endif
@@ -121,16 +124,17 @@ namespace snapper
     {
 	if (this != &dir)
 	{
+	    ::close(dirfd);
+	    dirfd = fcntl(dir.dirfd, F_DUPFD_CLOEXEC, 0);
+	    if (dirfd == -1)
+	    {
+		y2err("fcntl(F_DUPFD_CLOEXEC) failed error:" << stringerror(errno));
+		throw IOErrorException();
+	    }
+
 #ifdef ENABLE_XATTRS
 	    xastatus = dir.xastatus;
 #endif
-	    ::close(dirfd);
-	    dirfd = dup(dir.dirfd);
-	    if (dirfd == -1)
-	    {
-		y2err("dup failed" << " error:" << stringerror(errno));
-		throw IOErrorException();
-	    }
 	}
 
 	return *this;
@@ -146,11 +150,11 @@ namespace snapper
     SDir
     SDir::deepopen(const SDir& dir, const string& name)
     {
-       string::size_type pos = name.find('/');
-       if (pos == string::npos)
-	   return SDir(dir, name);
+	string::size_type pos = name.find('/');
+	if (pos == string::npos)
+	    return SDir(dir, name);
 
-       return deepopen(SDir(dir, string(name, 0, pos)), string(name, pos + 1));
+	return deepopen(SDir(dir, string(name, 0, pos)), string(name, pos + 1));
     }
 
 
@@ -185,10 +189,10 @@ namespace snapper
     vector<string>
     SDir::entries(entries_pred_t pred) const
     {
-	int fd = dup(dirfd);
+	int fd = fcntl(dirfd, F_DUPFD_CLOEXEC, 0);
 	if (fd == -1)
 	{
-	    y2err("dup failed" << " error:" << stringerror(errno));
+	    y2err("fcntl(F_DUPFD_CLOEXEC) failed error:" << stringerror(errno));
 	    throw IOErrorException();
 	}
 
@@ -410,18 +414,15 @@ namespace snapper
 	assert(path.find('/') == string::npos);
 	assert(path != "..");
 
-	int fd = ::openat(dirfd, path.c_str(), O_RDONLY | O_NOFOLLOW | O_NOATIME | O_CLOEXEC);
+	int fd = ::openat(dirfd, path.c_str(), O_RDONLY | O_NOFOLLOW | O_NONBLOCK | O_NOATIME |
+			  O_CLOEXEC);
 	if (fd >= 0)
 	{
 	    ssize_t r1 = ::flistxattr(fd, list, size);
-	    close(fd);
+	    ::close(fd);
 	    return r1;
 	}
-	else if (errno != ELOOP)
-	{
-	    return -1;
-	}
-	else
+	else if (errno == ELOOP || errno == ENXIO || errno == EWOULDBLOCK)
 	{
 	    boost::lock_guard<boost::mutex> lock(cwd_mutex);
 
@@ -436,6 +437,10 @@ namespace snapper
 	    chdir("/");
 	    return r2;
 	}
+	else
+	{
+	    return -1;
+	}
     }
 
 
@@ -445,18 +450,15 @@ namespace snapper
 	assert(path.find('/') == string::npos);
 	assert(path != "..");
 
-	int fd = ::openat(dirfd, path.c_str(), O_RDONLY | O_NOFOLLOW | O_NOATIME | O_CLOEXEC);
+	int fd = ::openat(dirfd, path.c_str(), O_RDONLY | O_NOFOLLOW | O_NONBLOCK | O_NOATIME |
+			  O_CLOEXEC);
 	if (fd >= 0)
 	{
 	    ssize_t r1 = ::fgetxattr(fd, name, value, size);
-	    close(fd);
+	    ::close(fd);
 	    return r1;
 	}
-	else if (errno != ELOOP)
-	{
-	    return -1;
-	}
-	else
+	else if (errno == ELOOP || errno == ENXIO || errno == EWOULDBLOCK)
 	{
 	    boost::lock_guard<boost::mutex> lock(cwd_mutex);
 
@@ -471,6 +473,10 @@ namespace snapper
 	    chdir("/");
 	    return r2;
 	}
+	else
+	{
+	    return -1;
+	}
     }
 
 
@@ -488,7 +494,8 @@ namespace snapper
 	    }
 	    else
 	    {
-                y2err("Couldn't get extended attributes status for " << base_path << "/" << path << stringerror(errno));
+                y2err("Couldn't get extended attributes status for " << base_path << "/" <<
+		      path << stringerror(errno));
                 throw IOErrorException();
 	    }
 	}
